@@ -57,6 +57,7 @@ gw_raise_error (GWLangArena arena, const char *proc, const char *fmt, ...)
   
   gw_lang->raise_error (arena, proc, message);
   
+  /* FIXME: we leak 'message' here, since this line won't be reached */
   free (message);
 }
 
@@ -126,8 +127,6 @@ gw_wrapset_new (GWLangArena arena,
   
   ws->registered = 0;
 
-  ws->lang_data = NULL;
-  
   return ws;
 }
 
@@ -192,7 +191,6 @@ gw_wrapset_add_type (GWWrapSet *ws,
   ti->wrap_value = wrap_value;
   ti->unwrap_value = unwrap_value;
   ti->destruct_value = destruct_value;
-  ti->lang_data = NULL;
   
   ws->types_sorted = 0;
 }
@@ -266,7 +264,6 @@ gw_wrapset_add_function (GWWrapSet *ws,
   fi->generic_name = generic_name;
   fi->arg_types = NULL;
   fi->ret_type = NULL;
-  fi->lang_data = NULL;
   
   /* FIXME: FFI should be able to handle optional args */
   assert (!arg_types || !n_optional_args);
@@ -307,7 +304,7 @@ gw_wrapset_add_function (GWWrapSet *ws,
   
     if (fi->n_req_args > 0)
     {
-        /* Data is used by ffi_call, so don't free it */
+      /* Data is used by ffi_call, so don't free it */
       arg_ffi = (ffi_type **) gw_malloc (ws->arena,
                                          sizeof (ffi_type *) * fi->n_req_args);
       for (i = 0; i < fi->n_req_args; i++)
@@ -323,11 +320,22 @@ gw_wrapset_add_function (GWWrapSet *ws,
                              fi->ret_type->type, arg_ffi);
       assert (status == FFI_OK);
       
+      /* note that we treat the space for the rvalue specially; libffi
+       * says it might need up to sizeof(ffi_arg) bytes of
+       * word-aligned memory for it, so we put it first (so it should
+       * be aligned on word boundary) and make it at least
+       * sizeof(ffi_arg) bytes. */
+      fi->data_area_size += (fi->ret_type->type->size > sizeof(ffi_arg)
+                             ? fi->ret_type->type->size : sizeof(ffi_arg));
+      
       /* now we know the sizes of the types and calculate the data
        * area size where we store the arguments' values */
       for (i = 0; i < fi->n_req_args; i++)
+      {
+        fi->data_area_size = GW_ALIGN(fi->data_area_size,
+                                      arg_ffi[i]->alignment);
         fi->data_area_size += arg_ffi[i]->size;
-      fi->data_area_size += fi->ret_type->type->size;
+      }
     }
   }
   
