@@ -26,6 +26,12 @@ USA.
 extern "C" {
 #endif
 
+#if defined(__GNUC__)
+#  define GW_NORETURN __attribute__ ((noreturn))
+#else
+#  define GW_NORETURN
+#endif
+
 typedef void *GWLangLocative;
 typedef void *GWLangArena;
 typedef enum _GWErrorStatus GWErrorStatus;
@@ -60,22 +66,22 @@ struct _GWLangSupport
     void *(*realloc) (GWLangArena arena, void *mem, size_t size);
     void  (*raise_error) (GWLangArena arena,
                           const char *func_name,
-                          const char *error);
+                          const char *error) GW_NORETURN;
     void  (*handle_wrapper_error) (GWLangArena arena,
                                    GWError *error,
                                    const char *func_name,
-                                   unsigned int arg_pos);
+                                   unsigned int arg_pos) GW_NORETURN;
 };
 
 int        gw_runtime_init (GWLangSupport *lang);
 void *     gw_malloc (GWLangArena arena, size_t size);
 void *     gw_realloc (GWLangArena arena, void *mem, size_t size);
 void       gw_raise_error (GWLangArena arena,
-                           const char *proc, const char *fmt, ...);
+                           const char *proc, const char *fmt, ...) GW_NORETURN;
 void       gw_handle_wrapper_error (GWLangArena arena,
                                     GWError *error,
                                     const char *func_name,
-                                    unsigned int arg_pos);
+                                    unsigned int arg_pos) GW_NORETURN;
 
 /*
  * Runtime information (RTI) section
@@ -88,11 +94,12 @@ typedef unsigned long GWTypeSpec;
 
 enum
 {
-  GW_TYPESPEC_CALLER_OWNED = 0x01,
-  GW_TYPESPEC_CALLEE_OWNED = 0x02,
-  GW_TYPESPEC_NULL_OK      = 0x04,
-  GW_TYPESPEC_CONST        = 0x08,
-  GW_TYPESPEC_OUT          = 0x10
+  GW_TYPESPEC_CALLER_OWNED  = 0x01,
+  GW_TYPESPEC_CALLEE_OWNED  = 0x02,
+  GW_TYPESPEC_NULL_OK       = 0x04,
+  GW_TYPESPEC_CONST         = 0x08,
+  GW_TYPESPEC_OUT           = 0x10,
+  GW_TYPESPEC_UNSPECIALIZED = 0x20
 };
 
 #define GW_TYPESPEC_USER_SHIFT 8
@@ -116,14 +123,12 @@ struct _GWTypeInfo
 {
     const char *name;
     const char *class_name;
-
+  
     ffi_type *type; /* if non-NULL, this type may passed dynamically */
 
     GWWrapValueFunc wrap_value;
     GWUnWrapValueFunc unwrap_value;
     GWDestructValueFunc destruct_value;
-    
-    void *lang_data;  /* Can be freely used by the language runtime */
 };
 
 struct _GWFunctionInfo
@@ -145,8 +150,6 @@ struct _GWFunctionInfo
 
     ffi_cif cif;
     unsigned int data_area_size; /* Size needed for the dynamic-call info */
-
-    void *lang_data;  /* Can be freely used by the language runtime */
 };
 
 struct _GWWrapSet
@@ -169,8 +172,6 @@ struct _GWWrapSet
     int nfuncs_allocated;
     int types_sorted : 1;
     int registered : 1;
-
-    void *lang_data;  /* Can be freely used by the language runtime */
 };
 
 GWWrapSet *gw_wrapset_new (GWLangArena arena,
